@@ -1,23 +1,8 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2016  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2016  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,19 +17,14 @@
 #include <errno.h>
 
 #include "private.h"
-#include "util.h"
+#include "useful.h"
 #include "key.h"
 #include "string.h"
 #include "random.h"
+#include "missing.h"
 
 #ifndef KEYCTL_DH_COMPUTE
 #define KEYCTL_DH_COMPUTE 23
-
-struct keyctl_dh_params {
-	int32_t private;
-	int32_t prime;
-	int32_t base;
-};
 #endif
 
 #ifndef KEYCTL_PKEY_QUERY
@@ -79,6 +59,20 @@ struct keyctl_pkey_params {
 	};
 	uint32_t __spare[7];
 };
+
+/* Work around the missing (pre-4.7) or broken (4.14.{70,71,72} and
+ * 4.18.{8,9,10}) kernel declaration of struct keyctl_dh_params
+ */
+struct dh_params {
+	int32_t private;
+	int32_t prime;
+	int32_t base;
+};
+#else
+/* When KEYCTL_PKEY_QUERY is defined by the kernel, the
+ * struct keyctl_dh_params declaration is valid.
+ */
+#define dh_params keyctl_dh_params
 #endif
 
 #ifndef KEYCTL_RESTRICT_KEYRING
@@ -99,6 +93,7 @@ struct l_keyring {
 static const char * const key_type_names[] = {
 	[L_KEY_RAW] = "user",
 	[L_KEY_RSA] = "asymmetric",
+	[L_KEY_ECC] = "asymmetric",
 };
 
 static long kernel_add_key(const char *type, const char *description,
@@ -130,11 +125,11 @@ static long kernel_update_key(int32_t serial, const void *payload, size_t len)
 	return result >= 0 ? result : -errno;
 }
 
-static long kernel_revoke_key(int32_t serial)
+static long kernel_invalidate_key(int32_t serial)
 {
 	long result;
 
-	result = syscall(__NR_keyctl, KEYCTL_REVOKE, serial);
+	result = syscall(__NR_keyctl, KEYCTL_INVALIDATE, serial);
 
 	return result >= 0 ? result : -errno;
 }
@@ -201,7 +196,9 @@ static long kernel_dh_compute(int32_t private, int32_t prime, int32_t base,
 {
 	long result;
 
-	struct keyctl_dh_params params = { private, prime, base };
+	struct dh_params params = { .private = private,
+				    .prime = prime,
+				    .base = base };
 
 	result = syscall(__NR_keyctl, KEYCTL_DH_COMPUTE, &params, payload, len,
 			NULL);
@@ -276,6 +273,7 @@ LIB_EXPORT struct l_key *l_key_new(enum l_key_type type, const void *payload,
 {
 	struct l_key *key;
 	char *description;
+	static unsigned long key_idx;
 
 	if (unlikely(!payload))
 		return NULL;
@@ -288,7 +286,7 @@ LIB_EXPORT struct l_key *l_key_new(enum l_key_type type, const void *payload,
 
 	key = l_new(struct l_key, 1);
 	key->type = type;
-	description = l_strdup_printf("ell-key-%p", key);
+	description = l_strdup_printf("ell-key-%lu", key_idx++);
 	key->serial = kernel_add_key(key_type_names[type], description, payload,
 					payload_length, internal_keyring);
 	l_free(description);
@@ -313,7 +311,12 @@ LIB_EXPORT void l_key_free(struct l_key *key)
 	if (unlikely(!key))
 		return;
 
-	kernel_revoke_key(key->serial);
+	/*
+	 * Use invalidate as, unlike revoke, this doesn't delay the
+	 * key garbage collection and causes the quota used by the
+	 * key to be released sooner and more predictably.
+	 */
+	kernel_invalidate_key(key->serial);
 
 	l_free(key);
 }
@@ -350,7 +353,7 @@ LIB_EXPORT bool l_key_extract(struct l_key *key, void *payload, size_t *len)
 	keylen = kernel_read_key(key->serial, payload, *len);
 
 	if (keylen < 0 || (size_t)keylen > *len) {
-		memset(payload, 0, *len);
+		explicit_bzero(payload, *len);
 		return false;
 	}
 
@@ -365,18 +368,16 @@ LIB_EXPORT ssize_t l_key_get_payload_size(struct l_key *key)
 
 static const char *lookup_cipher(enum l_key_cipher_type cipher)
 {
-	const char* ret = NULL;
-
 	switch (cipher) {
 	case L_KEY_RSA_PKCS1_V1_5:
-		ret = "pkcs1";
-		break;
+		return "pkcs1";
 	case L_KEY_RSA_RAW:
-		ret = "raw";
-		break;
+		return "raw";
+	case L_KEY_ECDSA_X962:
+		return "x962";
 	}
 
-	return ret;
+	return NULL;
 }
 
 static const char *lookup_checksum(enum l_checksum_type checksum)
@@ -466,7 +467,7 @@ LIB_EXPORT struct l_key *l_key_generate_dh_private(const void *prime_buf,
 	buf[0] |= 1 << ((prime_bits - 2) % 8);
 
 	private = l_key_new(L_KEY_RAW, buf, private_bytes);
-	memset(buf, 0, private_bytes);
+	explicit_bzero(buf, private_bytes);
 	l_free(buf);
 	return private;
 }
@@ -483,9 +484,8 @@ static bool compute_common(struct l_key *base, struct l_key *private,
 	if (result_len > 0) {
 		*len = result_len;
 		return usable_payload;
-	} else {
-		return false;
 	}
+	return false;
 }
 
 LIB_EXPORT bool l_key_compute_dh_public(struct l_key *generator,
@@ -515,13 +515,13 @@ static int be_bignum_compare(const uint8_t *a, size_t a_len,
 				return 1;
 
 		return memcmp(a + i, b, b_len);
-	} else {
-		for (i = 0; i < b_len - a_len; i++)
-			if (b[i])
-				return -1;
-
-		return memcmp(a, b + i, a_len);
 	}
+
+	for (i = 0; i < b_len - a_len; i++)
+		if (b[i])
+			return -1;
+
+	return memcmp(a, b + i, a_len);
 }
 
 /*
@@ -642,12 +642,13 @@ LIB_EXPORT struct l_keyring *l_keyring_new(void)
 {
 	struct l_keyring *keyring;
 	char *description;
+	static unsigned long keyring_idx;
 
 	if (!internal_keyring && !setup_internal_keyring())
 		return NULL;
 
 	keyring = l_new(struct l_keyring, 1);
-	description = l_strdup_printf("ell-keyring-%p", keyring);
+	description = l_strdup_printf("ell-keyring-%lu", keyring_idx++);
 	keyring->serial = kernel_add_key("keyring", description, NULL, 0,
 						internal_keyring);
 	l_free(description);
@@ -700,7 +701,7 @@ LIB_EXPORT void l_keyring_free(struct l_keyring *keyring)
 	if (unlikely(!keyring))
 		return;
 
-	kernel_revoke_key(keyring->serial);
+	kernel_invalidate_key(keyring->serial);
 
 	l_free(keyring);
 }
